package com.nova.flutter_ads;

import android.app.Activity;
import android.os.Handler;
import android.os.Looper;

import com.ark.adkit.basics.configs.Strategy;
import com.ark.adkit.basics.utils.JsonUtils;
import com.ark.adkit.polymers.polymer.ADTool;
import com.ark.adkit.polymers.polymer.utils.ArkVideoCallbacks;


import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BasicMessageChannel;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.StringCodec;

/**
 * FlutterAdsPlugin
 */
public class FlutterAdsPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    private BasicMessageChannel rewardRunTimeSender;
    private BasicMessageChannel insertRunTimeSender;
    private static FlutterPluginBinding pluginBinding;
    private static Activity activity;
    private static Registrar mRegistrar;


    public static boolean needPermission = true;
    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        mRegistrar = registrar;
        FlutterAdsPlugin flutterAdsPlugin = new FlutterAdsPlugin();
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_ads");
        channel.setMethodCallHandler(flutterAdsPlugin);
        activity = registrar.activity();
        setupViews(registrar);
    }


    @Override
    public void onMethodCall(MethodCall call, Result result) {
        if (call.method.equals("initRewardVideo")) {
            initRewardAd(call, result);
        } else if (call.method.equals("initInsertVideo")) {
            initInsertAd(call, result);
        } else if (call.method.equals("setConfig")){


            Object permission = call.argument("needPermission");
            if (permission!=null){
                needPermission = (boolean) permission;
            }
            ADTool.Builder builder = new ADTool.Builder();
            Object bannerWidth = call.argument("bannerWidth");
            Object nativeWidth = call.argument("nativeWidth");
            if (bannerWidth!=null){
                builder.setTtadBannerWidth((Integer) bannerWidth);
            }
            if (nativeWidth!=null){
                builder.setTtadBannerWidth((Integer) nativeWidth);
            }
            ADTool.initialize(builder.build());
        }else {
            result.notImplemented();
        }
    }

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        pluginBinding = flutterPluginBinding;
        final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "flutter_ads");
        channel.setMethodCallHandler(new FlutterAdsPlugin());

    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {

    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        activity = binding.getActivity();
        setupViews();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {
    }

    private static void setupViews() {

        //注册开屏广告UI插件
        SplashViewFactory factory = new SplashViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/splashview", factory);

        //注册本地广告
        NativeViewFactory nativeViewFactory = new NativeViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/nativeview", nativeViewFactory);

        //注册banner广告
        BannerViewFactory bannerViewFactory = new BannerViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/bannerview", bannerViewFactory);
    }

    private static void setupViews(Registrar registrar) {
        //注册开屏广告UI插件
        SplashViewFactory factory = new SplashViewFactory(registrar.activity(), registrar.messenger());
        registrar.platformViewRegistry().registerViewFactory("plugins.nova.ads/splashview", factory);

        //注册本地广告
        NativeViewFactory nativeViewFactory = new NativeViewFactory(registrar.activity(), registrar.messenger());
        registrar.platformViewRegistry().registerViewFactory("plugins.nova.ads/nativeview", nativeViewFactory);

        //注册banner广告
        BannerViewFactory bannerViewFactory = new BannerViewFactory(registrar.activity(), registrar.messenger());
        registrar.platformViewRegistry().registerViewFactory("plugins.nova.ads/bannerview", bannerViewFactory);
    }


    Handler handler;

    private void initInsertAd(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        BinaryMessenger binaryMessenger = null;
        if (pluginBinding != null) {
            binaryMessenger = pluginBinding.getBinaryMessenger();
        } else if (mRegistrar != null) {
            binaryMessenger = mRegistrar.messenger();
        }
        insertRunTimeSender = new BasicMessageChannel<>(binaryMessenger, "plugins.nova.ads.event/insert", StringCodec.INSTANCE);
        ADTool.getADTool().getManager().getVideoWrapper().loadFull(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onAdClose() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {

            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }

    private void initRewardAd(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        BinaryMessenger binaryMessenger = null;
        if (pluginBinding != null) {
            binaryMessenger = pluginBinding.getBinaryMessenger();
        } else if (mRegistrar != null) {
            binaryMessenger = mRegistrar.messenger();
        }

        rewardRunTimeSender = new BasicMessageChannel<>(binaryMessenger, "plugins.nova.ads.event/reward", StringCodec.INSTANCE);

        ADTool.getADTool().getManager().getVideoWrapper().loadReward(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onAdClose() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {

            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }
}
