import 'dart:async';
import 'dart:io';

import 'package:flutter/services.dart';

typedef RewardResult = void Function(bool result);
typedef InsertResult = void Function(bool result);

class FlutterAds {
  static const MethodChannel _channel = const MethodChannel('flutter_ads');

  static const BasicMessageChannel<dynamic> rewardMessageChannel =
      const BasicMessageChannel('plugins.nova.ads.event/reward', StringCodec());

  static const BasicMessageChannel<dynamic> insertMessageChannel =
      const BasicMessageChannel('plugins.nova.ads.event/insert', StringCodec());

  static var maxBannerHeight = 130.0;

  static var maxNativeHeight = 350.0;

  static void setConfig(
      {bool isNeedPermission = true,
      int bannerWidth = 360,
      int nativeWidth = 360,
      double bannerHeight,
      double nativeHeight}) async {
    var map = {
      "needPermission": isNeedPermission,
      "bannerWidth": bannerWidth,
      "nativeWidth": nativeWidth,
    };

    if (bannerHeight != null) {
      maxBannerHeight = bannerHeight;
    }
    if(nativeHeight !=null){
      maxNativeHeight = nativeHeight;
    }

    _channel.invokeMethod('setConfig', map);
  }

  static void initRewardVideo({RewardResult callback}) {
    _channel.invokeMethod('initRewardVideo');
    rewardMessageChannel.setMessageHandler((value) {
      if (value == "finish") {
        callback(true);
      } else {
        callback(false);
      }
    });
  }

  static void initInsertVideo({InsertResult callback}) async {
    _channel.invokeMethod('initInsertVideo');
    insertMessageChannel.setMessageHandler((value) {
      if (value == "finish") {
        callback(true);
      } else {
        callback(false);
      }
    });
  }
}
